@@ -3,7 +3,7 @@
  *	(Part of SKS cryptosystem)
  *
  * 
- * 	Copyright (C) 2004-2006  Manuel Pancorbo Castro
+ * 	Copyright (C) 2004-2007  Manuel Pancorbo Castro
  * 
  *	This program is free software; you can redistribute it and/or
  *	modify it under the terms of the GNU General Public License
@@ -33,14 +33,15 @@ int ecCheck (/*const*/ ec_point p)
 {
 	gf_poly t1, t2, t3;
 	
-	assert(ecExists(p));
+	assert(p != NULL);
+	//assert(ecExists(p));
 	#ifdef EC_A
 	gf_poly a;
-	gfOpen(a);
+	//gfOpen(a);
 	gfSet(a, EC_A);
 	#endif
 	
-	gfOpen(t3); gfOpen(t1); gfOpen(t2);
+	//gfOpen(t3); gfOpen(t1); gfOpen(t2);
 
 	gfSquare (t1, p->y);
 	gfMultiply (t2, p->x, p->y);
@@ -62,56 +63,60 @@ int ecCheck (/*const*/ ec_point p)
 	return ret;
 } /* ecCheck */
 
+#if 0
 ec_point ec_Open()
 {
 	ec_point a;
 
 	if( (a = malloc(sizeof(_ecpoint))) == NULL ) return NULL;
-	gfOpen(a->x);
+	//gfOpen(a->x);
 	if( a->x == NULL ) return NULL;
-	gfOpen(a->y);
+	//gfOpen(a->y);
 	if( a->y == NULL ) return NULL;
 	
 	return a;
 }
 
-void ecClear(ec_point a)
-{
-	assert(a != NULL);
-	gfClear(a->x);
-	gfClear(a->y);
-	free(a);
-}
-
 ec_projective ec_ProjOpen()
 {
 	ec_projective a;
 
 	if( (a = malloc(sizeof(_ecprojective))) == NULL ) return NULL;
-	gfOpen(a->x);
+	//gfOpen(a->x);
 	if( a->x == NULL ) return NULL;
-	gfOpen(a->y);
+	//gfOpen(a->y);
 	if( a->y == NULL ) return NULL;
-	gfOpen(a->z);
+	//gfOpen(a->z);
 	if( a->z == NULL ) return NULL;
 	
 	return a;
 }
 
+#endif
+
+void ecClear(ec_point a)
+{
+	assert(a != NULL);
+	gfClear(a->x);
+	gfClear(a->y);
+	//free(a);
+}
+
 void ecProjClear(ec_projective a)
 {
 	assert(a != NULL);
 	gfClear(a->x);
 	gfClear(a->y);
 	gfClear(a->z);
-	free(a);
+	//free(a);
 }
 
+
 int ecEqual (/*const*/ ec_point p, /*const*/ ec_point q)
 	/* evaluates to 1 if p == q, otherwise 0 (or an error code) */
 {
 	assert( (p != NULL) && (q != NULL) );
-	assert(ecExists(p) && ecExists(q));
+	//assert(ecExists(p) && ecExists(q));
 	return gfEqual (p->x, q->x) && gfEqual (p->y, q->y);
 } /* ecEqual */
 
@@ -119,7 +124,7 @@ void ecZero(ec_point p)
 	/* sets p to the point at infinity O, clearing entirely the content of p */
 {
 	assert(p != NULL);
-	assert(ecExists(p));
+	//assert(ecExists(p));
 	gfZero (p->x);
 	gfZero (p->y);
 } /* ecZero */
@@ -129,7 +134,7 @@ void ecCopy (ec_point p, /*const*/ ec_point q)
 	/* sets p := q */
 {
 	assert( (p != NULL) && (q != NULL) );
-	assert(ecExists(p) && ecExists(q));
+	//assert(ecExists(p) && ecExists(q));
 	gfCopy (p->x, q->x);
 	gfCopy (p->y, q->y);
 } /* ecCopy */
@@ -138,7 +143,7 @@ void ecProj2Affine(ec_point p, /* const */ ec_projective q)
 	/* Changes projective coordinates (q) to affine (p) p <- q */
 {
 	assert( (p != NULL) && (q != NULL) );
-	assert(ecExists(p) && ecProjExists(q));
+	//assert(ecExists(p) && ecProjExists(q));
 	
 	gf_poly z2;	
 	if(gf_iszero(q->z)){	/** Infinite point **/
@@ -146,7 +151,8 @@ void ecProj2Affine(ec_point p, /* const */ ec_projective q)
 		gfZero(p->y);
 	}
 	else{
-		gfOpen(z2); gfSquare(z2, q->z);
+		//gfOpen(z2); 
+		gfSquare(z2, q->z);
 		gfDivide(p->x, q->x, q->z);
 		gfDivide(p->y, q->y, z2);
 		gfClear(z2);
@@ -157,7 +163,7 @@ void ecAffine2Proj(ec_projective q, /* const */ ec_point p )
 	/* Changes affine coordinates (p) to projective (q) q <- p */
 {
 	assert( (p != NULL) && (q != NULL) );
-	assert(ecExists(p) && ecProjExists(q));
+	//assert(ecExists(p) && ecProjExists(q));
 	gfCopy(q->x, p->x);
 	gfCopy(q->y, p->y);
 	gfSet(q->z, 1);
@@ -170,8 +176,8 @@ ERROR ecCalcY (ec_point p, int ybit)
 	gf_poly a, b, t;
 	
 	assert(p != NULL);
-	assert(ecExists(p));
-	gfOpen(a); gfOpen(b); gfOpen(t);
+	//assert(ecExists(p));
+	//gfOpen(a); gfOpen(b); gfOpen(t);
 	gfSet(b, EC_B);
 	if(gf_iszero(p->x)){
 		/* elliptic equation reduces to y^2 = EC_B: */
@@ -228,7 +234,7 @@ void ecRandom (ec_point p)
 {
 	int check;
 	assert(p != NULL);
-	assert(ecExists(p));
+	//assert(ecExists(p));
 	do {
 		/* generate a pseudo-random x component: */
 		gfRandom (p->x);
@@ -252,11 +258,11 @@ int ecYbit (/*const*/ ec_point p)
 {
 	gf_poly t1, t2;
 	assert(p != NULL);
-	assert(ecExists(p));
-	if (mp_iszero(p->x)) {
+	//assert(ecExists(p));
+	if (gf_iszero(p->x)) {
 		return 0;
 	} else {
-		gfOpen(t1); gfOpen(t2);
+		//gfOpen(t1); gfOpen(t2);
 		/*gfInvert (t1, p->x);
 		gfMultiply (t2, p->y, t1);*/
 		gfDivide(t2, p->y, p->x);
@@ -269,11 +275,11 @@ void ecProjDouble (ec_projective p)
 	/* sets p := 2*p in projective coordinates */
 {
 	assert(p != NULL);
-	assert(ecProjExists(p));
-	if(mp_iszero(p->x) && mp_iszero(p->y) /*&& mp_iszero(p->z)*/) return;
+	//assert(ecProjExists(p));
+	if(gf_iszero(p->x) && gf_iszero(p->y) /*&& gf_iszero(p->z)*/) return;
 
 	gf_poly z2, bz4;
-	gfOpen(z2); gfOpen(bz4);
+	//gfOpen(z2); gfOpen(bz4);
 	
 	gfSquare(z2, p->z);
 	gfSquare(p->x, p->x);
@@ -301,7 +307,7 @@ void ecProjAdd (ec_projective p, /*const*/ ec_point q)
 	/* sets p := p + q in projective coordinates. It is assumed q.z = 1 */
 {
 	assert( (p != NULL) && (q != NULL) );
-	assert(ecProjExists(p) && ecExists(q));
+	//assert(ecProjExists(p) && ecExists(q));
 	
 	/* first check if there is indeed work to do (q == 0): */
 	if (gf_iszero(q->x) && gf_iszero(q->y)) return;
@@ -314,7 +320,7 @@ void ecProjAdd (ec_projective p, /*const*/ ec_point q)
 	
 	gf_poly A, B, z2, C, D, E, F, G;
 	
-	gfOpen(A); gfOpen(B); gfOpen(z2);
+	//gfOpen(A); gfOpen(B); gfOpen(z2);
 	gfSquare(z2, p->z);
 	gfMultiply(A, q->y, z2);
 	gfAdd(A, A, p->y);	/* A = Y1 Z0^2 + Y0 */
@@ -333,7 +339,7 @@ void ecProjAdd (ec_projective p, /*const*/ ec_point q)
 		}
 	} 
 	else{
-		gfOpen(C); gfOpen(D);
+		//gfOpen(C); gfOpen(D);
 		gfMultiply(C, p->z, B);	/* C = B Z0 */
 		#ifdef EC_A /** A == 1 **/
 		gfAdd(D, C, z2);
@@ -345,13 +351,13 @@ void ecProjAdd (ec_projective p, /*const*/ ec_point q)
 		
 		gfSquare(p->z, C);	/* Z2 = C^2 */
 		
-		E = z2;
+//		E = z2;
 		gfMultiply(E, A, C); /* E = A C */
 		gfSquare(p->x, A);
 		gfAdd(p->x, p->x, D);
 		gfAdd(p->x, p->x, E);	/* X2 = A^2 + D + E */
 		
-		F = A; G = B;
+		//F = A; G = B;
 		gfMultiply(F, q->x, p->z);
 		gfAdd(F, F, p->x);	/* F = X2 + X1 Z2 */
 		
@@ -365,6 +371,7 @@ void ecProjAdd (ec_projective p, /*const*/ ec_point q)
 	}
 	
 	gfClear(A); gfClear(B); gfClear(z2);
+	gfClear(E);gfClear(G);gfClear(F);
 }
 
 void ecDouble (ec_point p)
@@ -374,12 +381,12 @@ void ecDouble (ec_point p)
 	/*gf_poly a;*/
 	/*short int A;*/
 	assert(p != NULL);
-	assert(ecExists(p));
-	if(mp_iszero(p->x) && mp_iszero(p->y)) return;
+	//assert(ecExists(p));
+	if(gf_iszero(p->x) && gf_iszero(p->y)) return;
 	/*a->K = 1; a->P[0] =  A = EC_A; */
 	/* evaluate lambda = x + y/x: */
 	
-	gfOpen(lambda); gfOpen(t1); gfOpen(t2);
+	//gfOpen(lambda); gfOpen(t1); gfOpen(t2);
 	
 	/*gfInvert (t1, p->x);
 	gfMultiply (lambda, p->y, t1);*/
@@ -406,11 +413,11 @@ void ecAdd (ec_point p, /*const*/ ec_point q)
 	/* sets p := p + q */
 {
 	assert( (p != NULL) && (q != NULL) );
-	assert(ecExists(p) && ecExists(q));
+	//assert(ecExists(p) && ecExists(q));
 	
 	/* first check if there is indeed work to do (q != 0): */
-	if (!mp_iszero(q->x) || !mp_iszero(q->y)) {
-		if (!mp_iszero(p->x) || !mp_iszero(p->y)) {
+	if (!gf_iszero(q->x) || !gf_iszero(q->y)) {
+		if (!gf_iszero(p->x) || !gf_iszero(p->y)) {
 			/* p != 0 and q != 0 */
 			if (gfEqual (p->x, q->x)) {
 				/* either p == q or p == -q: */
@@ -426,8 +433,8 @@ void ecAdd (ec_point p, /*const*/ ec_point q)
 			else {
 				/* p != 0, q != 0, p != q, p != -q */
 				gf_poly lambda, t, tx, ty, x3;
-				gfOpen(lambda); gfOpen(t); gfOpen(tx);
-				gfOpen(ty); gfOpen(x3);
+				//gfOpen(lambda); gfOpen(t); gfOpen(tx);
+				//gfOpen(ty); gfOpen(x3);
 				/* evaluate lambda = (y1 + y2)/(x1 + x2): */
 				gfAdd (ty, p->y, q->y);
 				gfAdd (tx, p->x, q->x);
@@ -470,16 +477,17 @@ static int diff_vector(signed char *b, vl_number x )
 	int old_bit, actual_bit, i = 0;
 	vl_number w;
 	
-	vlOpen(w); vlCopy(w, x);
-	actual_bit = old_bit = mp_isodd(w);
+	//vlOpen(w); 
+	vlCopy(w, x);
+	actual_bit = old_bit = gf_one(w);
 	b[i++] = -actual_bit;
 	
 	do{
-		mp_div_2(w, w);
-		actual_bit = mp_isodd(w);
+		gf2m_Div_x(w, w);
+		actual_bit = gf_one(w);
 		b[i++] = old_bit - actual_bit;
 		old_bit = actual_bit;
-	} while(!mp_iszero(w));
+	} while(!gf_iszero(w));
 	vlClear(w);
 		
 	return i;
@@ -497,24 +505,26 @@ void ecMultiply (ec_point p, vl_number k)
 #define MWIN_SIZE	8	/** Number of points of given representation **/
 {
 	assert( (p != NULL) && (k != NULL) );
-	assert(ecExists(p) && vlExists(k));
+	//assert(ecExists(p) && vlExists(k));
 	
-	if(mp_iszero(k)){
+	if(gf_iszero(k)){
 		ecZero(p);
 		return;
 	}
 	
 	_ecpoint pos_buf[MWIN_SIZE],	/** Table for positives 1,3,...2M-1 **/
 	         neg_buf[MWIN_SIZE];	/** Table for negatives -1,-3,...-(2M-1) **/
-	ec_point pos = pos_buf, neg = neg_buf, pp;
+	_ecpoint *pos = pos_buf, *neg = neg_buf;
+	ec_point pp;
 	register int i, j, t, v, last_one = 0, length;
 	const int m2 = 2*MWIN_SIZE;
 	
 	/*** Precalculate values ***/
-	ecOpen(pp); ecCopy(pp, p); ecDouble(pp);
+	//ecOpen(pp); 
+	ecCopy(pp, p); ecDouble(pp);
 	for(i = 0; i < MWIN_SIZE; ++i, ++pos, ++neg){
-		gfOpen(pos->x); gfOpen(neg->x);
-		gfOpen(pos->y); gfOpen(neg->y);
+		//gfOpen(pos->x); gfOpen(neg->x);
+		//gfOpen(pos->y); gfOpen(neg->y);
 		if(!i){ 
 			ecCopy(pos, p);
 		}
@@ -535,7 +545,8 @@ void ecMultiply (ec_point p, vl_number k)
 	/*** start! ***/
 	ec_projective r;
 	
-	ecProjOpen(r); gfZero(r->x); gfZero(r->y);
+	//ecProjOpen(r); 
+	gfZero(r->x); gfZero(r->y);
 	gfSet(r->z, 1);
 	pos = pos_buf; neg = neg_buf;
 	
@@ -571,6 +582,7 @@ void ecMultiply (ec_point p, vl_number k)
 	
 	ecProjClear(r);
 	memset(b, 0, GF_M + 1);
+	
 	for(i = 0; i < MWIN_SIZE; ++i, ++pos, ++neg){
 		gfClear(pos->x);
 		gfClear(neg->x);
@@ -589,15 +601,15 @@ void ecPack (/*const*/ ec_point p, ecPoint k )
 	/* packs ec. point into array (compressed point) */
 {
 	assert( (p != NULL) && (k != NULL) );
-	assert(ecExists(p) );
+	//assert(ecExists(p) );
 	gf_poly tmp;
 	
 	memset(k, 0, GF_SIZE);
-	if(mp_iszero(p->x))	return;
+	if(gf_iszero(p->x))	return;
 	
-	gfOpen(tmp);
+	//gfOpen(tmp);
 	gfCopy(tmp, p->x);
-	mp_mul_2(tmp, tmp);
+	gf2m_Mul_x(tmp, tmp);
 	*(tmp->dp) |= ecYbit(p);
 	
 	gfPack(tmp, k);
@@ -608,12 +620,12 @@ ERROR ecUnpack ( ec_point p, /*const*/ ecPoint k )
 	/* unpacks compressed ec. point from array into full ec. point */
 {
 	assert((p != NULL) && (k != NULL));
-	assert(ecExists(p));
+	//assert(ecExists(p));
 	int ybit;
 	
 	gfUnpack(p->x, k);
 	ybit = gfYbit(p->x);
-	mp_div_2(p->x, p->x);
+	gf2m_Div_x(p->x, p->x);
 	return ecCalcY(p, ybit);
 }
 
@@ -634,7 +646,7 @@ void ecSub (ec_point p, /*const*/ ec_point r)
 	/* sets p := p - r */
 {
 	ec_point t;
-	ecOpen(t);
+	//ecOpen(t);
 
 	gfCopy (t->x, r->x);
 	gfAdd  (t->y, r->x, r->y);
@@ -658,9 +670,9 @@ main(int argc, char ** argv)
 	else test_count = 10;
 	gfInit();
 	srand ((unsigned)(time(NULL) % 65521U));
-	ecOpen(f); ecOpen(g);
-	ecOpen(x); ecOpen(y);
-	ecProjOpen(xproj);
+	//ecOpen(f); ecOpen(g);
+	//ecOpen(x); ecOpen(y);
+	//ecProjOpen(xproj);
 	
 	i = 0;
 	ecZero(f);
@@ -669,7 +681,7 @@ main(int argc, char ** argv)
 	} while(ecCalcY(f, 0));
 	fprintf(stderr, "Primer elemento de curva:\n");
 	gfPrint(stderr, "F.x", f->x);
-	gfPrint(stderr, "F.x", f->y);
+	gfPrint(stderr, "F.y", f->y);
 	/*printf ("Tamaño ulong64: %d\n", sizeof(ulong64));*/
 	printf ("Executing %d curve self tests GF(2^%d)...\n",
 	 test_count, GF_M);
@@ -743,7 +755,7 @@ main(int argc, char ** argv)
 			printf ("Quadruplication test #%d failed!\n", i); 
 		}
 		/* scalar multiplication commutativity test: m*(n*f) = n*(m*f) */
-		vlOpen(n); vlOpen(m);
+		//vlOpen(n); vlOpen(m);
 		vlRandom(n); vlRandom(m);
 		/*vlSet(n, 266); vlSet(m, 2);*/
 		/*gfPrint(stderr, "F.x", f->x);*/
