@@ -3,7 +3,7 @@
  *
  *
  * 
- * 	Copyright (C) 2004-2006  Manuel Pancorbo Castro
+ * 	Copyright (C) 2004-2007  Manuel Pancorbo Castro
  * 
  *	This program is free software; you can redistribute it and/or
  *	modify it under the terms of the GNU General Public License
@@ -33,61 +33,32 @@
 #define _GFLIB_C
 #include "gflib.h"
 
-static gf_poly R /*, ci, fi, gi, ti*/;
+static gf_poly R;
 static int GF_Init = 0;
 
-/*unsigned long int  sqrsqr[256];*/  /** a^4 **/
-/*
-#ifdef FULLTABLES
-static unsigned short int tab[256][256];
-#else
-static unsigned short int tab[16][16];
-#endif
+//#if (defined FULLTABLES) && ((GF_DIGIT_BIT)==28)
+//	#define GF_SQR_TABLE_SIZE (1<<14)
+//	gf_unit  sqr[GF_SQR_TABLE_SIZE];
+//#else
+#define GF_SQR_TABLE_SIZE (1 << 8)
+static unsigned short int  sqr[GF_SQR_TABLE_SIZE];
+//#endif
 
-static unsigned short int tab[16][16];
-*/
+#if !(defined GF_TM1) && (defined FULLTABLES)
+
+static _gf_poly quad[GF_M];
 
-#if (defined FULLTABLES)  && (DIGIT_BIT==28)
-gf_unit  sqr[1<<14];
-#else
-static unsigned short int  sqr[256];
 #endif
 
 static int tables_builded = 0;
 
 static void build_tables(void)
 {
-	long int i, ii, jj;
+	gf_unit i, ii, jj;
 	gf_unit c;
-	#if (defined FULLTABLES)  && (DIGIT_BIT==28)
-	long int lim = (1 << 14);
-	#else
-	long int lim = 256;
-	#endif
-	/*
-	#ifdef FULLTABLES
-	int lim = 256;
-	#else
-	int lim = 16;
-	#endif
-	*/
-	#if 0
-	int lim = 16;
-
-	for(i = 0; i < lim; ++i){
-		for(j = 0; j < lim; ++j){
-			c = 0; ii = i; jj = j;
-			while(ii){
-				if(ii & 1) c ^= jj;
-				ii >>= 1;
-				jj <<= 1;
-			}
-			tab[i][j] = c;
-			/*fprintf(stderr, "%x,", c);*/
-		}
-		/*fprintf(stderr, "\n");*/
-	}
-	#endif
+	gf_poly a;
+	
+	gf_unit lim = GF_SQR_TABLE_SIZE;
 	
 	for(i = 0; i < lim; ++i){
 		/** Sqr **/
@@ -99,54 +70,59 @@ static void build_tables(void)
 		}
 		sqr[i] = c;
 	}
-	#if 0
-	for(i = 0; i < 256; ++i){
-		c = sqr[i];
-		sqrsqr[i] = (sqr[c >> 8] << 16) | sqr[c & 0xff] ;
+#if !(defined GF_TM1) && (defined FULLTABLES)
+	/* QuadSolve quick table */
+	tables_builded = 1;
+	gf2m_Set(a, 1);
+	for(i = 0; i < GF_M; ++i){
+		if(i == GF_TM0)	gf2m_Zero(&quad[i]);
+		else gf2m_SlowQuadSolve(&quad[i],a);
+		
+		gf2m_Mul_x(a,a);
 	}
-	#endif
-	/*fprintf(stderr, "sqr: %d\n", sqr[255]);*/
+	
+#endif
 }
 
-
-/*int gf2m_Open(gf_poly *a)*/
+/*
 gf_poly gf2m_Open()
 {
 	gf_poly a;
 	
 	if( (a = malloc(sizeof(mp_int))) == NULL ) return NULL;
-	if(mp_init_size(a, 2*GF_M / DIGIT_BIT + 4)) return NULL;
+	if(mp_init_size(a, 2*GF_M / GF_DIGIT_BIT + 4)) return NULL;
 	return a;
 }
-
+*/
 ERROR gf2m_Init(void)
 /* Sets global 'R' with reduction trinomial */
 /* Sets auxiliar variables */
 {
 	if(GF_Init) return 0;
-	
-	gfOpen(R);
+	GF_Init = 1;
+	//gfOpen(R);
 	if(R == NULL) return 1;
 	gf2m_Set(R, 1);
-	mp_mul_2d(R, GF_M - GF_T, R);
-	mp_add_d(R, 1, R);
-	mp_mul_2d(R, GF_T, R);
-	mp_add_d(R, 1, R);
+	gf2m_Mul_xn(R, GF_M - GF_T, R);
+	fp_add_d(R, 1, R);
+	gf2m_Mul_xn(R, GF_T, R);
+	fp_add_d(R, 1, R);
 	
-
+		
 	if(!tables_builded){
 		build_tables();
 
 		tables_builded = 1;
 	}
-
-	GF_Init = 1;
+	
+	
 	return 0;
 }
 
 void gf2m_Quit(void)
 {
-	gfClear(R);
+
+	//gfClear(R);
 	
 	GF_Init = 0;
 }
@@ -154,15 +130,15 @@ void gf2m_Quit(void)
 int gf2m_Deg(gf_poly a)
 {
 	register int deg;
-	register mp_digit c;
-	int m = USED(a);
+	register gf_unit c;
+	int m = a->used;
 	
 	assert(a != NULL);
-	assert(gfExists(a));
-	if(mp_iszero(a)) return -1;
+	//assert(gfExists(a));
+	if(gf_iszero(a)) return -1;
 	/*while(!DIGIT(a, m - 1)) m--;*/
-	c = DIGIT(a, m-1);
-	deg = (m-1) * DIGIT_BIT;
+	c = a->dp[m-1];
+	deg = (m-1) * GF_DIGIT_BIT;
 	while(c){
 		++deg;
 		c >>= 1;
@@ -171,13 +147,17 @@ int gf2m_Deg(gf_poly a)
 
 }
 
+#if 0
 void gf2m_Clear(gf_poly a)
 {	
-	assert( a != NULL );
-	assert(gfExists(a));
-	gf2m_Zero(a); mp_clear(a);
-	free(a);
+	return;
+	//assert( a != NULL );
+	//assert(gfExists(a));
+	//gf2m_Zero(a); 
+	//mp_clear(a);
+	//free(a);
 }
+#endif
 
 void gf2m_Pack(gf_poly a, gfPoint b)
 {
@@ -190,7 +170,7 @@ void gf2m_Pack(gf_poly a, gfPoint b)
 void gf2m_SmallAdd(gf_poly a, gf_unit b)
 {
 	assert( (a != NULL) );
-	assert( gfExists(a) );
+	//assert( gfExists(a) );
 	
 	if(!b) return;
 	
@@ -200,35 +180,62 @@ void gf2m_SmallAdd(gf_poly a, gf_unit b)
 	}
 	else{
 		*(a->dp) ^= b;
-		mp_clamp(a);
+		gf_clamp(a);
 	}
 }
 
+void gf2m_Add(gf_poly c, gf_poly a, gf_poly b)
+{
+	int  ix, px;
+	_gf_poly *x;
+	gf_poly t;
+	
+	if (a->used > b->used) {
+		gfCopy(t, a);
+		px = b->used;
+    	x = b;
+    } else {
+    	gfCopy(t, b);
+		px = a->used;
+    	x = a;
+    }
+    for (ix = 0; ix < px; ix++) {
+    	t->dp[ix] ^= x->dp[ix];
+  	}
+  	gf_clamp (t);
+  	gfCopy(c, t);
+    
+}
 
-#if (GF_T < DIGIT_BIT)
-void gf2m_Reduce(gf_poly p)
-	/* reduces p mod the irreducible trinomial X^191 + Y^9 + 1 */
+
+//#if (sizeof(fp_digit) > GF_T)
+inline void gf2m_Reduce_Small_T(gf_poly p)
+	/* reduces p mod the irreducible trinomial X^191 + X^9 + 1 */
 {
 	assert(GF_Init);
 	assert(p != NULL);
-	assert(gfExists(p));
+	//assert(gfExists(p));
 	
-	const int digit_offset = GF_M/DIGIT_BIT;
-	const int bit_offset_M0 = GF_M % DIGIT_BIT;
-	const int bit_offset_MT = (GF_M-GF_T) % DIGIT_BIT;
-	register mp_digit val, *ptr, mask;
+	const int digit_offset = GF_M/GF_DIGIT_BIT;
+	const int bit_offset_M0 = GF_M % GF_DIGIT_BIT;
+	const int bit_offset_MT = (GF_M-GF_T) % GF_DIGIT_BIT;
+	register gf_unit val, *ptr /*, mask*/;
+	gf_word mask;
 	
 	ptr = &(p->dp[p->used-1]);
 	
 	while(p->used > digit_offset + 1){
 		if((val = *ptr)){
+		/*
 			mask = (val >> bit_offset_MT);
 			mask ^= (val >> bit_offset_M0);
 			*(ptr - digit_offset) ^= mask;
-			mask = (val << (DIGIT_BIT - bit_offset_MT));
-			mask ^= (val << (DIGIT_BIT - bit_offset_M0));
-			mask &= MP_MASK;
-			*(ptr - digit_offset -1) ^= mask;
+		*/
+			mask = ((gf_word) val << (GF_DIGIT_BIT - bit_offset_MT));
+			mask ^= ((gf_word) val << (GF_DIGIT_BIT - bit_offset_M0));
+			//mask &= GF_MASK;
+			*(ptr - digit_offset -1) ^= (gf_unit) (mask & GF_MASK);
+			*(ptr - digit_offset) ^= (gf_unit) (mask >> GF_DIGIT_BIT);
 			*ptr = 0;
 		}
 		ptr--;
@@ -236,9 +243,9 @@ void gf2m_Reduce(gf_poly p)
 	}
 	
 	if(p->used == digit_offset + 1){
-		mask = ((1 << bit_offset_M0) - 1) ^ MP_MASK;
+		mask = (((gf_unit)1 << bit_offset_M0) - 1) ^ GF_MASK;
 		if((val = *ptr & mask)){
-			*ptr &= (mask ^ MP_MASK);
+			*ptr &= (mask ^ GF_MASK);
 			mask = (val >> bit_offset_MT);
 			mask ^= (val >> bit_offset_M0);
 			*(ptr - digit_offset) ^= mask;
@@ -247,19 +254,19 @@ void gf2m_Reduce(gf_poly p)
 	while(!(*ptr) && p->used){ptr--, p->used--;}	
 }
 
-#elif (GF_T % DIGIT_BIT == 0)
-void gf2m_Reduce(gf_poly p)
-	/* reduces p mod the irreducible trinomial X^191 + Y^140 + 1 */
+//#elif (GF_T % GF_DIGIT_BIT == 0)
+void gf2m_Reduce_Congruent_T(gf_poly p)
+	/* reduces p mod the irreducible trinomial X^191 + X^(GF_DIGIT_BIT*n) + 1 */
 {
 	assert(GF_Init);
 	assert(p != NULL);
-	assert(gfExists(p));
+	//assert(gfExists(p));
 
-	const int digit_offset_M0 = GF_M/DIGIT_BIT;
-	const int digit_offset_MT = (GF_M-GF_T) / DIGIT_BIT;
-	const int bit_offset_M0 = GF_M % DIGIT_BIT;
+	const int digit_offset_M0 = GF_M/GF_DIGIT_BIT;
+	const int digit_offset_MT = (GF_M-GF_T) / GF_DIGIT_BIT;
+	const int bit_offset_M0 = GF_M % GF_DIGIT_BIT;
 	
-	register mp_digit val, *ptr, mask;
+	register gf_unit val, *ptr, mask;
 	
 	ptr = &(p->dp[p->used-1]);
 	while(p->used > digit_offset_M0 + 1){
@@ -267,7 +274,7 @@ void gf2m_Reduce(gf_poly p)
 			mask = (val >> bit_offset_M0);
 			*(ptr - digit_offset_M0) ^= mask;
 			*(ptr - digit_offset_MT) ^= mask;
-			mask = (val << (DIGIT_BIT - bit_offset_M0)) & MP_MASK;
+			mask = (val << (GF_DIGIT_BIT - bit_offset_M0)) & GF_MASK;
 			*(ptr - digit_offset_M0 -1) ^= mask;
 			*(ptr - digit_offset_MT -1) ^= mask;
 			*ptr = 0;
@@ -276,9 +283,9 @@ void gf2m_Reduce(gf_poly p)
 		p->used--;
 	}
 	if(p->used == digit_offset_M0 + 1){
-		mask = ((1 << bit_offset_M0) - 1) ^ MP_MASK;
+		mask = (((gf_unit)1 << bit_offset_M0) - 1) ^ GF_MASK;
 		if((val = *ptr & mask)){
-			*ptr &= (mask ^ MP_MASK);
+			*ptr &= (mask ^ GF_MASK);
 			mask = (val >> bit_offset_M0);
 			*(ptr - digit_offset_M0) ^= mask;
 			*(ptr - digit_offset_MT) ^= mask;
@@ -287,20 +294,20 @@ void gf2m_Reduce(gf_poly p)
 	}
 	while(!(*ptr) && p->used){ptr--, p->used--;}	
 }
-#elif (GF_M - GF_T > DIGIT_BIT)
-void gf2m_Reduce(gf_poly p)
+//#elif (GF_M - GF_T > GF_DIGIT_BIT)
+void gf2m_Reduce_General(gf_poly p)
 	/* reduces p mod the irreducible trinomial X^191 + Y^T + 1 */
 {
 	assert(GF_Init);
 	assert(p != NULL);
-	assert(gfExists(p));
+	//assert(gfExists(p));
 
-	const int digit_offset_M0 = GF_M/DIGIT_BIT;
-	const int digit_offset_MT = (GF_M-GF_T) / DIGIT_BIT;
-	const int bit_offset_M0 = GF_M % DIGIT_BIT;
-	const int bit_offset_MT = (GF_M-GF_T) % DIGIT_BIT;
+	const int digit_offset_M0 = GF_M/GF_DIGIT_BIT;
+	const int digit_offset_MT = (GF_M-GF_T) / GF_DIGIT_BIT;
+	const int bit_offset_M0 = GF_M % GF_DIGIT_BIT;
+	const int bit_offset_MT = (GF_M-GF_T) % GF_DIGIT_BIT;
 	
-	register mp_digit val, *ptr, mask;
+	register gf_unit val, *ptr, mask;
 	
 	ptr = &(p->dp[p->used-1]);
 	while(p->used > digit_offset_M0 + 1){
@@ -309,9 +316,9 @@ void gf2m_Reduce(gf_poly p)
 			*(ptr - digit_offset_M0) ^= mask;
 			mask = (val >> bit_offset_MT);
 			*(ptr - digit_offset_MT) ^= mask;
-			mask = (val << (DIGIT_BIT - bit_offset_M0)) & MP_MASK;
+			mask = (val << (GF_DIGIT_BIT - bit_offset_M0)) & GF_MASK;
 			*(ptr - digit_offset_M0 -1) ^= mask;
-			mask = (val << (DIGIT_BIT - bit_offset_MT)) & MP_MASK;
+			mask = (val << (GF_DIGIT_BIT - bit_offset_MT)) & GF_MASK;
 			*(ptr - digit_offset_MT -1) ^= mask;
 			*ptr = 0;
 		}
@@ -319,9 +326,9 @@ void gf2m_Reduce(gf_poly p)
 		p->used--;
 	}
 	if(p->used == digit_offset_M0 + 1){
-		mask = ((1 << bit_offset_M0) - 1) ^ MP_MASK;
+		mask = (((gf_unit)1 << bit_offset_M0) - 1) ^ GF_MASK;
 		if((val = *ptr & mask)){
-			*ptr &= (mask ^ MP_MASK);
+			*ptr &= (mask ^ GF_MASK);
 			mask = (val >> bit_offset_M0);
 			*(ptr - digit_offset_M0) ^= mask;
 			mask = (val >> bit_offset_MT);
@@ -332,79 +339,145 @@ void gf2m_Reduce(gf_poly p)
 	while(!(*ptr) && p->used){ptr--, p->used--;}	
 }
 
-#else
-	#error "Unsupported trinomial"
+void gf2m_Reduce(gf_poly p)
+{
+	if(GF_T < GF_DIGIT_BIT) gf2m_Reduce_Small_T(p);
+	else if(GF_T % GF_DIGIT_BIT == 0) gf2m_Reduce_Congruent_T(p);
+	else if(GF_M - GF_T > GF_DIGIT_BIT) gf2m_Reduce_General(p);
+	return;
+}
 
-#endif
+//#else
+//	#error "Unsupported trinomial"
+
+//#endif
 
 
 
-mp_word gf2m_SmallMult(register gf_unit a, gf_unit b)
+#if 0
+inline gf_unit gf2m_HalfSmallMult(gf_unit a, gf_unit b)
+{
+	register gf_unit c = 0;
+	register gf_unit mdor, mando;
+	
+	mdor = (a < b) ? a : b;
+	mando = ( (a ^ b)) ^ mdor;
+	
+	while(mdor){
+		if(mdor & (gf_unit) 1) c ^= mando;
+		mdor >>= 1;	
+		mando <<= 1;
+	}
+	
+	return c;
+}
+
+inline gf_word gf2m_SmallMult(gf_unit a, gf_unit b)
 	/*** returns a*b ***/
 {
-	mp_word c = 0;
-	register gf_unit bit = (1 << (DIGIT_BIT - 1));
+	//if(!a || !b) return (gf_word) 0;
 	
-	while(bit){
-		c <<= 1;
-		if(a & bit) c ^= b;
-			
-		bit >>= 1;
+	gf_unit lo_a, hi_a, lo_b, hi_b;
+	static gf_unit shift2 = sizeof(gf_unit)/2;
+	//static gf_unit shift = sizeof(gf_unit);
+	static gf_unit mask = (1 << sizeof(gf_unit)/2) - 1;
+	gf_unit lo_c, hi_c;
+	gf_word c;
+	
+	lo_a = a & mask, lo_b = b & mask;
+	hi_a = a >> shift2, hi_b = b >> shift2;
+
+	lo_c = gf2m_HalfSmallMult(lo_a, lo_b);
+	hi_c = gf2m_HalfSmallMult(hi_a, hi_b);
+	c = (gf_word) hi_c << (gf_word) shift2;
+	c ^= gf2m_HalfSmallMult(hi_a^lo_a, hi_b^lo_b)^lo_c^hi_c;
+	c <<= (gf_word) shift2;
+	c ^= (gf_word)lo_c;
+	
+	return c;
+}
+#endif
+
+inline gf_word gf2m_SmallMult(gf_unit a, gf_unit b)
+	/*** returns a*b ***/
+{
+	gf_word c = 0;
+	gf_word mdor, mando;
+	
+	mdor =(gf_word) (a < b) ? a : b;
+	mando = ((gf_word) (a ^ b)) ^ mdor;
+	
+	while(mdor){
+		if(mdor & (gf_word) 1) c ^= mando;
+		mdor >>= 1;	
+		mando <<= 1;
 	}
 	
 	return c;
 }
 
 
+
 void _gf2m_Multiply (gf_poly s, /*const*/ gf_poly p, /*const*/ gf_poly q)
-	/** Sets r = p*q (no reduction) **/
-	/** It computes products of size DIGIT_BIT **/
+	/** Sets s = p*q (no reduction) **/
+	/** It computes products of size GF_DIGIT_BIT **/
 {
 	
 	int deg_p, deg_q, sum;
 	register int i, j, k;
-	mp_word c;
+	gf_word c, d[2*GF_M/GF_DIGIT_BIT+3];
 	gf_poly r;
 		
 	assert( (s != NULL) && (p != NULL) && (q != NULL) );
-	assert(gfExists(s) && gfExists(p) && gfExists(q));	
-	
-	
+	//assert(gfExists(s) && gfExists(p) && gfExists(q));	
+	while(p->used < q->used) p->dp[p->used++] = 0;
+	while(q->used < p->used) q->dp[q->used++] = 0;
 	deg_p = p->used;
 	deg_q = q->used;
 	sum = deg_p + deg_q - 1;
 	
-	gfOpen(r);
+	//gfOpen(r);
 	gf2m_Zero(r);
+	memset(d, 0, sizeof(d));
 	for(k = 0; k < sum; ++k){
-		
+		if(k < deg_p && k < deg_q)
+			d[k] = gf2m_SmallMult(p->dp[k], q->dp[k]);
 		c = 0;
 		i = k - deg_q + 1;
-		for(i = ((i < 0)? 0:i), j = k - i; (i < deg_p) && (j >= 0); ++i, --j)
-			c ^= gf2m_SmallMult(p->dp[i], q->dp[j]);
+		for(i = ((i < 0)? 0:i), j = k - i; (i < deg_p) && (j >= i); ++i, --j){
+			if(i == j){
+				 c ^= d[i];
+			}
+			else {
+				c ^= gf2m_SmallMult(p->dp[i] ^ p->dp[j], q->dp[i] ^ q->dp[j])^d[i]^d[j];
+			}
+		}
 		
-		r->dp[k] ^= (c & MP_MASK);
-		r->dp[k+1] ^= (c >> DIGIT_BIT);
+		r->dp[k] ^= (c & GF_MASK);
+		r->dp[k+1] ^= (c >> (gf_unit)GF_DIGIT_BIT);
 		r->used += 2;
-		/*mp_clamp(r);*/
+		/*gf_clamp(r);*/
 	}
-	r->used++; mp_clamp(r);
+	r->used++; 
+	gf_clamp(r);
 	gfCopy(s, r);
 	gfClear(r);
+	gf_clamp(p);
+	gf_clamp(q);
 }
 
-
 void gf2m_Multiply (gf_poly r, /*const*/ gf_poly p, /*const*/ gf_poly q)
 	/** Sets r = p*q (mod R) **/
 {
 	_gf2m_Multiply (r, p, q);
+	
 	gf2m_Reduce(r);
 }
 
-#if (defined FULLTABLES)  && (DIGIT_BIT==28)
+#if 0
 void _gf2m_Square(gf_poly r, /*const */ gf_poly p)
 	/* sets r = p^2 No reduction */
-	/* only if DIGIT_BIT == 28 **/
+	/* only if GF_DIGIT_BIT == 28 **/
 {
 	register int i;
 	register gf_unit c;
@@ -412,53 +485,69 @@ void _gf2m_Square(gf_poly r, /*const */ gf_poly p)
 	gf_poly s;
 	
 	assert( (r != NULL) && (p != NULL) );
-	assert(gfExists(r) && gfExists(p));
+	//assert(gfExists(r) && gfExists(p));
 	if(!tables_builded){
 		build_tables();
 		
 		tables_builded = 1;
 	}
-	gfOpen(s);gfZero(s);
+	//gfOpen(s);
+	gfZero(s);
 	for(i = 0; i < p->used; ++i){
 		c = p->dp[i];
 		s->dp[s->used++] = sqr[c & mask];
 		s->dp[s->used++] = sqr[c >> 14];
 	}
-	s->used++; mp_clamp(s);
-	gfCopy(r, s); gfClear(s);
+	s->used++; gf_clamp(s);
+	gfCopy(r, s); 
+	//gfClear(s);
 }
+
 #else
+
 void _gf2m_Square(gf_poly r, /*const */ gf_poly p)
-	/* sets r = p^2 No reduction */
 {
-	unsigned char buf[2*GF_SIZE];
-	int deg;
-	register int i;
+	register int i, j;
 	register gf_unit c;
+	int k, t;
+	gf_word d;
+	gf_poly s;
 	
 	assert( (r != NULL) && (p != NULL) );
-	assert(gfExists(r) && gfExists(p));
+	//assert(gfExists(r) && gfExists(p));
 	if(!tables_builded){
 		build_tables();
 		
 		tables_builded = 1;
 	}
-	
-	memset(buf, 0, 2*GF_SIZE);
-	deg = gfSize(p);
-	gfPut(p, buf);
-	
-	gf2m_Zero(r);
-	
-	for(i = 0; i < deg; ++i){
-		mp_mul_2d(r, 16, r);
-		c = sqr[buf[i]];
-		
-		gf2m_SmallAdd(r, c);
+	//gfOpen(s);
+	gfZero(s);
+	for(i = k = 0; i < p->used; ++i, k += 2){
+		c = p->dp[i];
 		
+		d = 0;
+		j = 0;
+		//fprintf(stderr, "Segmento %d:\t%lX\n", i, c);
+		while(c){
+			t= sqr[(int)(c & 0xff)];
+			//fprintf(stderr, "%04x ", t);
+			d ^= ((gf_word) t) << j;
+			j += 16;
+			//d <<= 16;
+			c >>= 8;
+			c &= GF_MASK;
+		}
+		//fprintf(stderr, "\n");
+		s->dp[k] ^= (d & GF_MASK);
+		s->dp[k+1] ^= (d >> (gf_unit)GF_DIGIT_BIT);
+		s->used += 2;
 	}
+	s->used++; gf_clamp(s);
+	gfCopy(r, s); 
+	gfClear(s);
 }
 
+
 #endif
 
 void gf2m_Square(gf_poly r, /*const */ gf_poly p)
@@ -469,116 +558,30 @@ void gf2m_Square(gf_poly r, /*const */ gf_poly p)
 }
 
 
-#ifdef GFLIB_TEST
-/*** Old inversion algorithm ***/
-ERROR gf2m_Divide2 (gf_poly b,/* const */ gf_poly p, /* const */ gf_poly a)
-	/* sets b := (p / a) mod (x^GF_M + x^GF_T + 1) */
-	/* warning: a, b and p must not overlap! */
-	/* 'p' can be null. Then, it will be performed a^-1 mod R */
-{
-	assert (GF_Init != 0);	
-	assert( (b != NULL) && (a != NULL) );
-	assert(gfExists(b) && gfExists(a));
-	assert ((b != a) && (p != a));
-	
-	if(mp_iszero(a)) return 1;
-	
-	register int j;
-	gf_poly ci, fi, gi, ti;
-	
-	gfOpen(ci); gfOpen(fi); gfOpen(gi);
-	gfOpen(ti);
-	/* initialize b := p; ci := 0; fi := a; gi := x^GF_M + x^GF_T + 1: */
-	if(p == NULL){ /** Interprets 'only inversion of a' **/
-		gf2m_Set(b, 1);
-	}
-	else {
-		gfCopy(b, p);
-	}
-	gf2m_Zero(ci);
-	gf2m_Copy(fi, a), gf2m_Copy(gi, R);
-	/*int count = 0;*/
-	
-	for(;;){
-		if(mp_cmp_d(fi, 1) == MP_EQ){
-			gfClear(ci), gfClear(fi);
-			gfClear(gi), gfClear(ti);
-			
-			if (p != NULL) gfReduce(b);
-			return 0;
-		}
-		if((j = gf2m_Deg(fi) - gf2m_Deg(gi)) < 0){
-			j = -j;
-			goto SWAP_FG; 
-		}
-SWAP_GF:
-		/*++count;*/
-		if(j){
-			mp_mul_2d(gi, j, ti);
-			gf2m_Add(fi, ti, fi);
-			mp_mul_2d(ci, j, ti);
-			
-			gf2m_Add(b, ti, b);
-		}
-		else{
-			gf2m_Add(fi, gi, fi);
-			gf2m_Add(b, ci, b);
-		}
-	}
-	for(;;){
-		if(mp_cmp_d(gi, 1) == MP_EQ){
-			if (p != NULL) gfReduce(ci);
-			gf2m_Copy(b, ci);
-			gfClear(ci), gfClear(fi);
-			gfClear(gi), gfClear(ti);
-			
-			return 0;
-		}
-		if((j = gf2m_Deg(gi) - gf2m_Deg(fi)) < 0){
-			j = -j;
-			goto SWAP_GF; 
-		}
-		
-SWAP_FG:
-		/*++count;*/
-		if(j){
-			mp_mul_2d(fi, j, ti);
-			gf2m_Add(gi, ti, gi);
-			mp_mul_2d(b, j, ti);
-			gf2m_Add(ci, ti, ci);
-		}
-		else{
-			gf2m_Add(gi, fi, gi);
-			gf2m_Add(ci, b, ci);
-		}
-	}
-}
-
-#endif
-
 /*** New inversion algorithm (a bit faster than previous ***/
-ERROR gf2m_Divide (gf_poly b,/* const */ gf_poly p, /* const */ gf_poly a)
+ERROR gf2m_Divide (gf_poly g1,/* const */ gf_poly p, /* const */ gf_poly a)
 	/* sets b := (p / a) mod (x^GF_M + x^GF_T + 1) */
 	/* warning: a, b and p must not overlap! */
 	/* 'p' can be null. Then, it will be performed a^-1 mod R */
 {
 	assert (GF_Init != 0);	
-	assert( (b != NULL) && (a != NULL) );
-	assert(gfExists(b) && gfExists(a));
-	assert ((b != a) && (p != a));
+	assert( (g1 != NULL) && (a != NULL) );
+	//assert(gfExists(b) && gfExists(a));
+	assert ((g1 != a) && (p != a));
 	
-	if(mp_iszero(a)) return 1;
+	if(gf_iszero(a)) return 1;
 	
-	const int M_digit_offset = (GF_M - 1)/ DIGIT_BIT,
-			  T_digit_offset = (GF_T - 1)/ DIGIT_BIT,
-			  M_bit_offset = (GF_M - 1) % DIGIT_BIT,
-			  T_bit_offset = (GF_T - 1) % DIGIT_BIT;
-	const gf_unit lo = (1 << T_bit_offset);
-	const gf_unit hi = (1 << M_bit_offset);
+	const int M_digit_offset = (GF_M - 1)/ GF_DIGIT_BIT,
+			  T_digit_offset = (GF_T - 1)/ GF_DIGIT_BIT,
+			  M_bit_offset = (GF_M - 1) % GF_DIGIT_BIT,
+			  T_bit_offset = (GF_T - 1) % GF_DIGIT_BIT;
+	const gf_unit lo = ((gf_unit)1 << T_bit_offset);
+	const gf_unit hi = ((gf_unit)1 << M_bit_offset);
 	int flag;
-	register gf_poly u, v, g2, g1 = b /** Alias of b **/;
+	/*register*/ gf_poly u, v, g2/*, g1*/; 
 	
-	gfOpen(g2); gfOpen(u); gfOpen(v);
+	//g1[0] = b[0] /** Alias of b **/;
+	//gfOpen(g2); gfOpen(u); gfOpen(v);
 	
 	/* initialize g1 := p; g2 := 0; u := a; v := x^GF_M + x^GF_T + 1: */
 	if(p == NULL){ /** Interprets 'only inversion of a' **/
@@ -591,28 +594,28 @@ ERROR gf2m_Divide (gf_poly b,/* const */ gf_poly p, /* const */ gf_poly a)
 	gf2m_Copy(u, a), gf2m_Copy(v, R);
 	/*int count = 0;*/
 
-	while( (mp_cmp_d(u, 1) != MP_EQ) && (mp_cmp_d(v, 1) != MP_EQ)){
-		while(!mp_isodd(u)){
-			mp_div_2(u, u);
-			flag = mp_isodd(g1);
-			mp_div_2(g1, g1);
+	while( (gf2m_CmpOne(u) != GF_EQ) && (gf2m_CmpOne(v) != GF_EQ)){
+		while(!gf_one(u)){
+			gf2m_Div_x(u, u);
+			flag = gf_one(g1);
+			gf2m_Div_x(g1, g1);
 			if(flag){
 				g1->dp[M_digit_offset] ^= hi;	
 				g1->dp[T_digit_offset] ^= lo;
 				g1->used = M_digit_offset + 1;
 			}
 		}
-		while(!mp_isodd(v)){
-			mp_div_2(v, v);
-			flag = mp_isodd(g2);
-			mp_div_2(g2, g2);
+		while(!gf_one(v)){
+			gf2m_Div_x(v, v);
+			flag = gf_one(g2);
+			gf2m_Div_x(g2, g2);
 			if(flag){
 				g2->dp[M_digit_offset] ^= hi;	
 				g2->dp[T_digit_offset] ^= lo;
 				g2->used = M_digit_offset + 1;
 			}
 		}
-		if(mp_cmp_mag(u, v) == MP_GT){
+		if(gf2m_Comp(u, v) == GF_GT){
 			gf2m_Add(u, u, v);
 			gf2m_Add(g1, g1, g2);
 		}
@@ -621,7 +624,7 @@ ERROR gf2m_Divide (gf_poly b,/* const */ gf_poly p, /* const */ gf_poly a)
 			gf2m_Add(g2, g1, g2);
 		}
 	}
-	if(mp_cmp_d(v, 1) == MP_EQ) gf2m_Copy(g1, g2);
+	if(gf2m_CmpOne(v) == GF_EQ) gf2m_Copy(g1, g2);
 	gfClear(g2), gfClear(u), gfClear(v);
 	
 	return 0;
@@ -635,14 +638,14 @@ void gf2m_SquareRoot (gf_poly p, gf_unit b)
 	gf_poly q;
 	
 	assert(p != NULL);
-	assert(gfExists(p)); 
+	//assert(gfExists(p)); 
 	assert (GF_Init != 0);
 	if(!b){
 		gf2m_Zero(p);
 		return;
 	}
 	
-	gfOpen(q);
+	//gfOpen(q);
 	
 	gf2m_Set(p, b);
 	i = GF_M - 1;
@@ -662,16 +665,18 @@ int gf2m_Trace(/*const*/ gf_poly p)
 
 	assert (GF_Init != 0);
 	assert( p != NULL );
-	assert(gfExists(p)); 
-	gfOpen(t);
+	//assert(gfExists(p)); 
+	//gfOpen(t);
 	
-	mp_div_2d(p, GF_TM0, t, NULL);
+	//mp_div_2d(p, GF_TM0, t, NULL);
+	gf2m_Div_xn(t, GF_TM0, p);
 	
-	tr ^= mp_isodd(t);
+	tr ^= gf_one(t);
 	#ifdef GF_TM1
-	while(!mp_iszero(t)){
-		mp_div_2d(t, GF_TM1, t, NULL);
-		tr ^= mp_isodd(t);
+	while(!gf_iszero(t)){
+		//mp_div_2d(t, GF_TM1, t, NULL);
+		gf2m_Div_xn(t, GF_TM1, t);
+		tr ^= gf_one(t);
 	}
 	#endif
 	gfClear(t);
@@ -680,19 +685,21 @@ int gf2m_Trace(/*const*/ gf_poly p)
 
 
 
-ERROR gf2m_QuadSolve (gf_poly p, /*const*/ gf_poly beta)
-	/* sets p to a solution of p^2 + p = beta */
+ERROR gf2m_SlowQuadSolve (gf_poly p, /*const*/ gf_poly beta)
+	/* sets p to a solution of p^2 + p = beta 
+		Slow version */
 {
-	register int i;
-	
+#if 0
 	assert (GF_Init != 0);
 	assert( (beta != NULL) && (p != NULL) );
-	assert(gfExists(beta) && gfExists(p));
+	//assert(gfExists(beta) && gfExists(p));
 	assert (p != beta);
 	if (gfTrace (beta) != 0) {
 		return 1; /* no solution */
 	}
-
+#endif	
+	register int i;
+	
 	gf2m_Copy (p, beta);
 	for (i = 0; i < GF_M/2; i++) {
 		gf2m_Square (p, p);
@@ -701,18 +708,58 @@ ERROR gf2m_QuadSolve (gf_poly p, /*const*/ gf_poly beta)
 	}
 	return 0;
 } /* gfQuadSolve */
-#endif
+
+
+ERROR gf2m_QuadSolve (gf_poly p, /*const*/ gf_poly beta)
+	/* sets p to a solution of p^2 + p = beta 
+		Quick version. It uses precalculated values of
+		beta = x^i */
+{
+
+	assert (GF_Init != 0);
+	assert( (beta != NULL) && (p != NULL) );
+	assert (p != beta);
+	if (gfTrace (beta) != 0) {
+		return 1; /* no solution */
+	}
+	
+#if !(defined GF_TM1) && (defined FULLTABLES)
+	register int i;
+	int j;
+	register gf_unit bit_check = 1;
+	
+	gf2m_Zero(p);
+	for(i = 0; i < GF_DIGIT_BIT; ++i){
+		for(j = 0; j < beta->used; ++j){
+			if(bit_check & beta->dp[j])
+				gf2m_Add(p, p, quad+GF_DIGIT_BIT*j+i);
+		}
+		bit_check <<= 1;
+	}
+	return 0;
+#else
+	return gf2m_SlowQuadSolve (p, beta);
+#endif /* GF_TM1 - FULLTABLES */
+}
+
+#endif /* ?TRACE_MASK */
 
 void gf2m_Random(gf_poly a)
 	/* Only for testing purposes */
 {
-	int i;
+	int i, shift;
+	unsigned int rnd;
+	
 	assert( a != NULL);
-	assert(gfExists(a)); 
+	//assert(gfExists(a)); 
 	gf2m_Zero(a);
-	for(i = 0; i < 192/DIGIT_BIT + 1; ++i){
-		mp_mul_2d(a, DIGIT_BIT, a);
-		mp_add_d(a, rand() & MP_MASK, a);
+	
+	shift = (sizeof(rnd) > GF_DIGIT_BIT)? GF_DIGIT_BIT: sizeof(rnd);
+	for(i = 0; i < GF_M/shift + 1; ++i){
+		gf2m_Mul_xn(a, shift, a);
+		//mp_mul_2d(a, shift, a);
+		rnd = rand();
+		fp_add_d(a, rnd & GF_MASK, a);
 	}
 	gf2m_Reduce(a);
 }
@@ -721,7 +768,7 @@ void gf2m_Print (FILE *fOut, /*const*/ char *tag, /*const*/ gf_poly p)
 	/* prints tag and the contents of p to file fOut */
 {
 	char buf[256];
-	mp_tohex(p, buf);
+	fp_toradix(p, buf,16);
 	fprintf (fOut, "%s = %s\n", tag, buf);
 
 }
@@ -735,7 +782,7 @@ int gf2m_SlowTrace (/*const*/ gf_poly p)
 	gf_poly t;
 
 	assert (GF_Init != 0);
-	gfOpen(t);
+	//gfOpen(t);
 	gf2m_Copy (t, p);
 	for (i = 1; i < GF_M; i++) {
 		gf2m_Square (t, t);
@@ -761,6 +808,7 @@ main(int argc, char ** argv)
 		,dfail = 0
 		,sfail = 0
 		,ifail = 0
+//		,kfail = 0
 		,rfail = 0
 		,tfail = 0
 		,qfail = 0
@@ -769,16 +817,20 @@ main(int argc, char ** argv)
 		;
 	gf_poly f, g, h, x, y, z;
 	gf_unit b;
-	clock_t elapsed, elap_quad = 0L, elap_mul = 0, 
-			elap_inv = 0, elap_inv2 = 0, elap_sqr = 0;
-	ulong64 TOGGLE = (1LL << DIGIT_BIT) - 1;
+	clock_t elapsed, elap_quad = 0L, elap_mul = 0, //elap_kar = 0, 
+			elap_inv = 0, //elap_inv2 = 0, 
+			elap_sqr = 0;
+	gf_word TOGGLE = ((gf_word)1 << GF_DIGIT_BIT) - 1;
 	
 	if(argc > 1){
 		test_count = atoi(argv[1]);
 	}
 	else test_count = 10;
 	
-	printf("Bits por dígito %d\n", DIGIT_BIT);
+	printf("Bits por dígito %d\n", GF_DIGIT_BIT);
+	printf("Máscara %ld\n", GF_MASK);
+	printf("Tamaño de dígito %d\n", sizeof(gf_unit));
+	printf("Tamaño de doble %d\n", sizeof(gf_word));
 	gfInit();
 	gfPrint(stderr, "R", R);
 	
@@ -786,8 +838,8 @@ main(int argc, char ** argv)
 	printf ("Executing %d field self tests.", test_count);
 	/*printf("Parameters:\n Curve: %d\nField (2^%d)(2^%d)\n", LaCurva, GF_L, GF_K);*/
 	printf("Bits: %d\nTrinomial: %d\n", GF_M, GF_T);
-	gfOpen(f); gfOpen(g); gfOpen(h);
-	gfOpen(x); gfOpen(y); gfOpen(z);
+	//gfOpen(f); gfOpen(g); gfOpen(h);
+	//gfOpen(x); gfOpen(y); gfOpen(z);
 	elapsed = -clock ();
 	
 	for (i = 0; i < test_count; i++) {
@@ -809,7 +861,7 @@ main(int argc, char ** argv)
 		
 		/* multiplication test: f*g = g*f */
 		/*mp_set(f, 1);
-		mp_clamp(f);*/
+		gf_clamp(f);*/
 		
 		
 		elap_mul -= clock();
@@ -817,7 +869,7 @@ main(int argc, char ** argv)
 		gf2m_Multiply (y, g, f);
 		elap_mul += clock();
 		if (!gfEqual (x, y)) {
-			fprintf(stderr, "Tamaño mp_word = %d\n", sizeof(mp_word));
+			fprintf(stderr, "Tamaño gf_word = %d\n", sizeof(gf_word));
 			gfPrint(stderr, "Verdadero", x);
 			gfPrint(stderr, "Prueba   ", y);
 			
@@ -826,7 +878,7 @@ main(int argc, char ** argv)
 			fprintf(stderr, "Elementos g: %d\n\n", g->K);*/
 			/* printf ("Multiplication test #%d failed!\n", i); */
 			/*fprintf(stderr, "Prueba de multiplicar: %lx x %lx = %llx\n",
-			b = (1 << DIGIT_BIT ) - 1;
+			b = (1 << GF_DIGIT_BIT ) - 1;
 				1, b, gf2m_SmallMult(b, 1));*/
 		}
 		
@@ -853,70 +905,96 @@ main(int argc, char ** argv)
 			/* printf ("Distribution test #%d failed!\n", i); */
 		}
 		
+		/* karatsuba mul. test */
+		#if 0
+		elap_mul -= clock();
+		gfMultiply (x, f, g);
+		elap_mul += clock();
+		elap_kar -= clock();
+		gf2m_KaratsubaMul(y, f, g);
+		gfReduce(y);
+		elap_kar += clock();
+		if (!gfEqual (x, y)) {
+			kfail++;
+			/*gfPrint(stderr, "f", f);
+			gfPrint(stderr, "g", g);
+			gfPrint(stderr, "h", h);
+			gfPrint(stderr, "f*(g+h)", x);
+			fprintf(stderr, "Elementos: %d\n\n", x->K);
+			gfPrint(stderr, "f*g+f*h", y);
+			fprintf(stderr, "Elementos: %d\n\n", y->K);
+			return -1; */
+			/* printf ("Distribution test #%d failed!\n", i); */
+		}
+		#endif
+		
 		/* squaring test: f^2 = f*f */
 		gfZero(x), gfZero(y);
-		/*mp_set(f, 0x1234567);
-		mp_mul_2d(f, 10, f);*/
+		gfSet(f, 0xA234567);
+		gf2m_Mul_xn(f, 100, f);
+		
+		gfReduce(f);
 		elap_sqr -= clock();
 		_gf2m_Square (x, f);
 		gfReduce(x);
 		elap_sqr += clock();
+		
 		elap_mul -= clock();
 		_gf2m_Multiply (y, f, f);
 		gfReduce(y);
 		elap_mul += clock();
+				
 		if (!gfEqual (x, y)) {
 			sfail++;
 			gfPrint(stderr, "f", f);
 			gfPrint(stderr, "f^2", x);
 			gfPrint(stderr, "f*f", y);
+			fprintf(stderr, "Número de dígitos: %d\n", x->used);
 			fprintf(stderr, "Último dígito: %lx\n", y->dp[y->used-1]);
 			fprintf(stderr, "Siguiente dígito: %lx\n", y->dp[y->used]);
 			/*return -1;*/
 			/* printf ("Squaring test #%d failed:\n", i); */
 		}
-		#if 0
-		/** Division test **/
-		mp_div_2d(f, GF_M/2, f, NULL);
-		gf2m_div(g, f, x, y);
-		gfPrint(stderr, "Dividendo", g);
-		gfPrint(stderr, "Divisor", f);
-		gfPrint(stderr, "Cociente", x);
-		gfPrint(stderr, "Resto", y);
-		gfMultiply(f, x, f);
-		gfAdd(f, y, f);
-		gfPrint(stderr, "Resultado", f);
-			
-		if(!gfEqual(f, g)){
-			dvfail++;
-			gfPrint(stderr, "Original", g);
-			gfPrint(stderr, "Resultado", f);
-		}
-		#endif
-		/* inversion test: g*(f/g) = f */
-		if (!mp_iszero(g)) {
+		
+		/** inversion test: g*(f/g) = f **/
+		if (!gf_iszero(g)) {
 			/*gf2m_Set(g, 2);*/
 			gfReduce(g); 
 			elap_inv -= clock();
-			gf2m_Divide (x, f, g); /* x = f/g */
+			gf2m_Divide (x, f, g); /* x = 1/g */
 			elap_inv += clock();
-			elap_inv2 -= clock();
-			gf2m_Divide2 (y, f, x); /* y = f/x = f*g/f = g */
-			elap_inv2 += clock();
+			//elap_inv2 -= clock();
+			//gf2m_Divide2 (y, f, x); /* y = f/x = f*g/f = g */
+			//elap_inv2 += clock();
 			/*gfReduce(f);*/
 			/*fprintf(stderr, "Grado de f: %d\n", gfDeg(f));*/
 			
-			/*gfMultiply(z, g, x); *//* z= x*g =? f */
-			if (!gfEqual (g, y)) {
-			/*
-				gfPrint(stderr, "Original", g);
-				gfPrint(stderr, "Resultado", x);
+			gfMultiply(y, g, x); /* y= x*g =? f */
+			if (!gfEqual (f, y)) {
+			
+				gfPrint(stderr, "Dividendo, f", f);
+				gfPrint(stderr, "Divisor, g", g);
+				gfPrint(stderr, "Cociente, x", x);
+				gfPrint(stderr, "g·x = f?", y);
 				ifail++;
-			*/
+			
 				printf ("Inversion test #%d failed!\n", i); 
 			}
 		}
-		/* square root test: sqrt(b)^2 = b */
+		#if 0
+		/** new square algorithm test **/
+		mp_div_2d(g, GF_M/2, g, NULL);
+		gfSquare(x, g);
+		_gf2m_Square_new(y, g);
+		gfReduce(y);
+		if(!gfEqual (x, y)) {
+			gfPrint(stderr, "Original", g);
+			gfPrint(stderr, "Cuadrado      ", x);
+			gfPrint(stderr, "Nuevo cuadrado", y);
+			return -1;
+		}
+		#endif
+		/** square root test: sqrt(b)^2 = b */
 		b =  rand () & TOGGLE;
 		gf2m_Zero(z);
 		if (b) {
@@ -926,6 +1004,8 @@ main(int argc, char ** argv)
 		}
 		gfSquareRoot (y, b);
 		gfSquare (x, y);
+		//_gf2m_Square_old(x, y);
+		//gf2m_Reduce(x);
 		
 		if (!gfEqual (x, z)) {
 			rfail++;
@@ -943,7 +1023,8 @@ main(int argc, char ** argv)
 			/* printf ("Trace test #%d failed!\n", i); */
 		}
 		
-		/* quadratic equation solution test: x^2 + x = f (where tr(f) = 0)*/
+		/** quadratic equation solution test: x^2 + x = f (where tr(f) = 0)*/
+		
 		if (gfTrace (f) == 0) {
 			/*fprintf(stderr, "Traza no nula\n");*/
 			elap_quad -= clock ();
@@ -958,9 +1039,6 @@ main(int argc, char ** argv)
 				/* printf ("Quadratic equation test #%d failed!\n", i); */
 			}
 		}
-		
-		
-	
 	}
 	elapsed += clock ();
 	
@@ -968,11 +1046,14 @@ main(int argc, char ** argv)
 	                   
 	
 	printf (" done, elapsed time = %.2f s.\n", (float)elapsed/CLOCKS_PER_SEC);
-	printf (" multiplication time = %.4f s.\n", (float)elap_mul/CLOCKS_PER_SEC/6.);
+	printf (" multiplication time = %.4f s.\n", (float)elap_mul/CLOCKS_PER_SEC/7.);
+	//printf (" karatsuba mult. time = %.4f s.\n", (float)elap_kar/CLOCKS_PER_SEC);
 	printf (" squaring time = %.4f s.\n", (float)elap_sqr/CLOCKS_PER_SEC/2.);
+	
 	printf (" quad- solving time = %.4f s.\n", (float)elap_quad/CLOCKS_PER_SEC);
+	
 	printf (" inversion time = %.4f s.\n", (float)elap_inv/CLOCKS_PER_SEC);
-	printf (" old inversion time = %.4f s.\n", (float)elap_inv2/CLOCKS_PER_SEC);
+	//printf (" old inversion time = %.4f s.\n", (float)elap_inv2/CLOCKS_PER_SEC);
 	if (gfail) printf ("---> %d degree checks failed <---\n", gfail);
 	if (afail) printf ("---> %d additions failed <---\n", afail);
 	if (mfail) printf ("---> %d multiplications failed <---\n", mfail);
@@ -982,7 +1063,7 @@ main(int argc, char ** argv)
 	if (rfail) printf ("---> %d square roots failed <---\n", rfail);
 	if (tfail) printf ("---> %d traces failed <---\n", tfail);
 	/*if (qfail) printf ("---> %d quadratic equations failed <---\n", qfail);*/
-	gfClear(x), gfClear(y), gfClear(z), gfClear(f), gfClear(g), gfClear(h); 
+	//gfClear(x), gfClear(y), gfClear(z), gfClear(f), gfClear(g), gfClear(h); 
 	return afail || mfail || dfail || dvfail || sfail || ifail || rfail || tfail || qfail;
 }
 
@@ -993,7 +1074,7 @@ main()
 	gf_poly x;
 	int tr, i;
 	
-	gfOpen(x);
+	//gfOpen(x);
 	gf2m_Set(x, 1);
 	
 	gfInit();
@@ -1002,10 +1083,11 @@ main()
 		/*tr = gf2m_SlowTrace(x);*/
 		if((tr = gf2m_SlowTrace(x)))
 			printf("Elemento: %d\tTraza: %d\n", i, tr);
-		mp_mul_2(x,x);
+		gf2m_Mul_x(x,x);
 	}
-	gfClear(x);
+	//gfClear(x);
 	gfQuit();
 }
+
 #endif
 
