@@ -1,5 +1,11 @@
 /*
 	SKS, by M.Pancorbo
+	version 0.93 Changed math lib to "tomfastmath" who gives full-word alignment
+				for the unit container (so, the number of bits/coefficients a unit
+				has is actually 32 or 64 instead of 28 or 60). General improvement of
+				'gflib' with noteworthy performance gain. Among others, Karatsuba's 
+				techniques was introduced. Also, its code was a bit cleaned up.
+	version 0.92c Corrected a bug when compiled in a 64-bit box.
 	version 0.92b Corrected a bug when trying to encrypt small files. Now, gcc
 				does not complain about rigourous warnings. 
 				Conversion UTF-8 -> iso-8859-1 has been avoided in linux
@@ -55,7 +61,7 @@
 		PRNG:		/dev/random, wincrypt or ANSI-RNG
 
 
-	Copyright (C) 2004-2006  Manuel Pancorbo Castro
+	Copyright (C) 2004-2007  Manuel Pancorbo Castro
 
 	This program is free software; you can redistribute it and/or
 	modify it under the terms of the GNU General Public License
@@ -193,7 +199,7 @@ ERROR do_signature(FILE *f_inp, FILE *f_out, FILE *f_key, char **pattern)
 			}
 		}
 		fprintf(stderr, "  [%08lx]\n", id);
-		/** Any way, go on **/
+		/** Anyway, go on **/
 		if(!match) fputs(msg_no_sign_key, stderr);
 	}
 	
